"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resync = exports.except = exports.affect = exports.state = exports.wait = void 0;
const execution_1 = require("./execution");
const interrupt_1 = require("./interrupt");
let queue = [];
function wait(provider) {
    if (queue.length) {
        return queue.shift().run();
    }
    throw new interrupt_1.AsyncInterrupt(provider());
}
exports.wait = wait;
function state(fn) {
    return wait(async () => fn());
}
exports.state = state;
function affect(fn) {
    state(fn);
}
exports.affect = affect;
function except(e) {
    if (e instanceof interrupt_1.AsyncInterrupt) {
        throw e;
    }
}
exports.except = except;
function resync(fn) {
    async function runEvaluation(fnQueue, ...params) {
        try {
            queue = [...fnQueue];
            const value = fn(...params);
            if (queue.length) {
                throw new Error('Resync evaluation differed between runs, make sure impure functions are called inside state()');
            }
            return value;
        }
        catch (e) {
            if (!(e instanceof interrupt_1.AsyncInterrupt)) {
                throw e;
            }
            await e.promise
                .then(v => fnQueue.push(new execution_1.SuccessExecution(v)))
                .catch(e => fnQueue.push(new execution_1.FailureExecution(e)));
            return await runEvaluation(fnQueue, ...params);
        }
    }
    return async (...params) => await runEvaluation([], ...params);
}
exports.resync = resync;
